@@ -4,8 +4,8 @@ import tempfile
 import email
 import getpass
 
-from ..externals import httplib2
-from ..externals import simplejson as json
+import httplib2
+import json
 
 from .select import Select
 from .cache import CacheManager, HTCache
@@ -15,6 +15,10 @@ from .uriutil import join_uri
 from .jsonutil import csv_to_json
 from .errors import is_xnat_error
 from .errors import catch_error
+from .array import ArrayData
+from .xpath_store import XpathStore
+from . import xpass
+
 
 DEBUG = False
 
@@ -78,10 +82,35 @@ v           config: string
                other parameters that might have been given.
         """
 
-        self._interactive = not all([server, user, password]) and not config
+        self._interactive = False
 
-        if config is not None:
+        if not all([server, user, password]) and not config:
+            self._interactive = True
+
+        if all(arg is None
+               for arg in [server, user, password, config]) \
+               and os.path.exists(xpass.path()):
+
+            connection_args = xpass.read_xnat_pass(xpass.path())
+
+            if connection_args is None:
+                raise Exception('XNAT configuration file not found '
+                                'or formated incorrectly.')
+
+            self._server = connection_args['host']
+            self._user = connection_args['u']
+            self._pwd = connection_args['p']
+            self._cachedir = os.path.join(
+                cachedir, '%s@%s' % (
+                    self._user, 
+                    self._server.split('//')[1].replace(
+                        '/', '.').replace(':', '_')
+                    )
+                )
+
+        elif config is not None:
             self.load_config(config)
+
         else:
             if server is None:
                 self._server = raw_input('Server: ')
@@ -122,8 +151,10 @@ v           config: string
 
         self.inspect = Inspector(self)
         self.select = Select(self)
+        self.array = ArrayData(self)
         self.cache = CacheManager(self)
         self.manage = GlobalManager(self)
+        self.xpath = XpathStore(self)
         
         if _DRAW_GRAPHS:
             self._get_graph = GraphData(self)
@@ -131,9 +162,6 @@ v           config: string
 
         if self._interactive:
             self._get_entry_point()
-            self._jsession = self._exec('%s/JSESSION' % self._entry)
-            if is_xnat_error(self._jsession):
-                catch_error(self._jsession)
 
         self.inspect()
 
@@ -141,12 +169,18 @@ v           config: string
         if self._entry is None:
             # /REST for XNAT 1.4, /data if >=1.5
             self._entry = '/REST'
-            try:
-                self._exec('/data/JSESSION', 'HEAD')
+            try:                
+                self._jsession = self._exec('/data/JSESSION')
                 self._entry = '/data'
-            except:
-                pass
+
+                if is_xnat_error(self._jsession):
+                    catch_error(self._jsession)
+            except Exception, e:
+                if not '/data/JSESSION' in e.message:
+                    raise e
             
+        return self._entry
+
     def _connect(self, **kwargs):
         """ Sets up the connection with the XNAT server.
 
@@ -163,9 +197,24 @@ v           config: string
         else:
             kwargs = self._connect_extras
         
+        kwargs['disable_ssl_certificate_validation'] = True
+
         if DEBUG:   
             httplib2.debuglevel = 2
-        self._http = httplib2.Http(HTCache(self._cachedir, self), **kwargs)
+
+        # compatibility with httplib2 < 0.7
+        try:
+            self._http = httplib2.Http(
+                HTCache(self._cachedir, self), 
+                **kwargs
+                )
+        except:
+            del kwargs['disable_ssl_certificate_validation']
+            self._http = httplib2.Http(
+                HTCache(self._cachedir, self), 
+                **kwargs
+                )
+            
         self._http.add_credentials(self._user, self._pwd)
 
     def _exec(self, uri, method='GET', body=None, headers=None):
@@ -194,6 +243,8 @@ v           config: string
 
         uri = join_uri(self._server, uri)
 
+        if DEBUG:
+            print uri
         # using session authentication
         headers['cookie'] = self._jsession
         headers['connection'] = 'keep-alive'
@@ -270,6 +321,9 @@ v           config: string
                                                            )
                                              )
 
+        if is_xnat_error(content):
+            catch_error(content)
+
         return content
 
 
@@ -296,14 +350,15 @@ v           config: string
                 uri += '?format=csv'
 
         content = self._exec(uri, 'GET')
-
+        
         if is_xnat_error(content):
             catch_error(content)
 
         return csv_to_json(content)
 
     def _get_head(self, uri):
-        print 'GET HEAD'
+        if DEBUG:
+            print 'GET HEAD'
         _nocache = httplib2.Http()
         _nocache.add_credentials(self._user, self._pwd)
 
@@ -346,7 +401,7 @@ v           config: string
         config = {'server':self._server, 
                   'user':self._user, 
                   'password':self._pwd,
-                  'cachedir':self._cachedir,
+                  'cachedir':os.path.split(self._cachedir)[0],
                   }
 
         json.dump(config, fp)
@@ -367,72 +422,23 @@ v           config: string
             config = json.load(fp)
             fp.close()
 
-            self._server = config['server']
-            self._user = config['user']
-            self._pwd = config['password']
-            self._cachedir = config['cachedir']
+            self._server = str(config['server'])
+            self._user = str(config['user'])
+            self._pwd = str(config['password'])
+            self._cachedir = str(config['cachedir'])
 
-#     def grab(self, datatype, seq_type=None):
-#         columns = []
-#         if datatype.endswith('ScanData'):
-#             columns = ['%s/%s' % (datatype, field)
-#                        for field in ['type', 'ID', 'image_session_ID']
-#                        ]
-# #        else:
-# #            columns = ['%s/%s'%(datatype, field) for field in ['type', 'ID', 'session_id']]
-#         try:
-#             data = self.select(datatype, columns).all()
-#         except:
-#             data = self.select(datatype).all()
+            self._cachedir = os.path.join(
+                self._cachedir, '%s@%s' % (
+                    self._user, 
+                    self._server.split('//')[1].replace(
+                        '/', '.').replace(':', '_')
+                    )
+                )
+        else:
+            raise Exception('Configuration file does not exists.')
 
-#         print data.headers()
+    def version(self):
+        return self._exec('/data/version')
 
-#         uris = []
-
-#         type_header = difflib.get_close_matches('type', data.headers())
-
-#         if difflib.get_close_matches('id', data.headers()) == []:
-#             session_header = difflib.get_close_matches('session_id', 
-#                                                        data.headers())[0]
-
-#             subject_header = difflib.get_close_matches('subject_id', 
-#                                                        data.headers())[0]
-
-#             project_header = difflib.get_close_matches('project', 
-#                                                        data.headers())[0]
-
-
-#             print project_header, subject_header, session_header
-
-#             for entry in data:
-#                 if seq_type is None \
-#                         or type_header == [] \
-#                         or entry[type_header[0]] == seq_type:
-
-#                     uris.append('/REST/projects/%s'
-#                                 '/subjects/%s'
-#                                 '/experiments/%s' % \
-#                                     (entry[project_header],
-#                                      entry[subject_header],
-#                                      entry[session_header]
-#                                      )
-#                                 )
-
-#         else:
-#             id_header = difflib.get_close_matches('id', data.headers())[0]
-#             session_header = difflib.get_close_matches('session_id', \
-#                                                            data.headers())[0]
-
-#             for entry in data:
-#                 if (seq_type is None 
-#                     or type_header == []
-#                     or entry[type_header[0]] == seq_type
-#                     ):
-                    
-#                     uris.append('/REST/experiments/%s/scans/%s' % \
-#                                     (entry[session_header], 
-#                                      entry[id_header]
-#                                      )
-#                                 )
-
-#         return CObject(uris, self)
+    def set_logging(self, level=0):
+        pass
